==> react-12.0.3.tgz: package/es/List/index.js <==
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

/** @component list-item */
import React from 'react';
import PropTypes from 'prop-types';
import omit from 'lodash/omit';
import uniqueId from 'lodash/uniqueId';
import qsa from 'dom-helpers/query/querySelectorAll';
import { UIDReset } from 'react-uid';
import SelectableContext from "../SelectableContext";
import ListContext from "../ListContext";

var List =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(List, _React$Component);

  List.getDerivedStateFromProps = function getDerivedStateFromProps(_ref, state) {
    var active = _ref.active;
    return active ? _extends({}, state, {
      listContext: _extends({}, state.listContext, {
        active: active
      })
    }) : state;
  };

  function List(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _this.determineInitialFocus = function () {
      var items = qsa(_this.listNode, ".cui-list-item:not(.disabled):not(:disabled):not(.cui-list-item--read-only)");
      _this._needsRefocus = true;
      items.length && _this.getNextFocusedChild(items[0], 0);
    };

    _this.getIncludesFirstCharacter = function (str, char) {
      return str.charAt(0).toLowerCase().includes(char);
    };

    _this.handleKeyDown = function (e) {
      var focus = _this.state.listContext.focus;
      var clickEvent;
      var tgt = e.currentTarget;
      var char = e.key;
      var items = qsa(_this.listNode, ".cui-list-item:not(.disabled):not(:disabled):not(.cui-list-item--read-only)");
      var length = items.length && items.length - 1 || 0;
      var focusIdx = focus && items.indexOf(_this.listNode.querySelector("[data-md-event-key=\"" + focus + "\"]")) || 0;
      var flag = false;

      var isPrintableCharacter = function isPrintableCharacter(str) {
        return str.length === 1 && str.match(/\S/);
      };

      switch (e.which) {
        case 32:
        case 13:
          try {
            clickEvent = new MouseEvent('click', {
              view: window,
              bubbles: true,
              cancelable: true
            });
          } catch (err) {
            if (document.createEvent) {
              // DOM Level 3 for IE 9+
              clickEvent = document.createEvent('MouseEvents');
              clickEvent.initEvent('click', true, true);
            }
          }

          tgt.dispatchEvent(clickEvent);
          flag = true;
          break;

        case 38:
        case 37:
          _this.getNextFocusedChild(tgt, -1);

          _this._needsRefocus = true;
          flag = true;
          break;

        case 39:
        case 40:
          _this.getNextFocusedChild(tgt, 1);

          _this._needsRefocus = true;
          flag = true;
          break;

        case 33:
        case 36:
          _this.setFocusToLimit('start', items, length);

          _this._needsRefocus = true;
          flag = true;
          break;

        case 34:
        case 35:
          _this.setFocusToLimit('end', items, length);

          _this._needsRefocus = true;
          flag = true;
          break;

        default:
          if (isPrintableCharacter(char)) {
            _this.setFocusByFirstCharacter(char, focusIdx, items, length);

            _this._needsRefocus = true;
            flag = true;
          }

          break;
      }

      if (flag) {
        e.stopPropagation();
        e.preventDefault();
      }
    };

    _this.handleSelect = function (e, opts) {
      var onSelect = _this.props.onSelect;
      var active = _this.state.listContext.active;
      var eventKey = opts.eventKey,
          label = opts.label,
          value = opts.value;
      var items = qsa(_this.listNode, '.cui-list-item');
      var index = items.indexOf(_this.listNode.querySelector("[data-md-event-key=\"" + eventKey + "\"]"));

      _this.setFocus(index); // Don't do anything if onSelect Event Handler is present


      if (onSelect) {
        return onSelect(e, {
          eventKey: items[index].attributes['data-md-event-key'].value,
          label: label,
          value: value
        });
      } // Don't do anything if index is the same or outside of the bounds


      if (eventKey === active || index < 0 || index > items.length - 1) return; // Keep reference to last index for event handler

      var last = active; // Call change event handler

      _this.setState(function (state) {
        return {
          last: last,
          listContext: _extends({}, state.listContext, {
            active: items[index].attributes['data-md-event-key'].value
          })
        };
      });
    };

    _this.setFocus = function (index) {
      var items = qsa(_this.listNode, '.cui-list-item');

      _this.setState(function (state) {
        return {
          listContext: _extends({}, state.listContext, {
            focus: items[index].attributes['data-md-event-key'].value
          })
        };
      });
    };

    _this.setFocusByFirstCharacter = function (char, focusIdx, items, length) {
      var newIndex = items.reduce(function (agg, item, idx, arr) {
        var index = focusIdx + idx + 1 > length ? Math.abs(focusIdx + idx - length) : focusIdx + idx + 1;
        return !agg.length && arr[index].attributes['data-md-keyboard-key'] && arr[index].attributes['data-md-keyboard-key'].value && _this.getIncludesFirstCharacter(arr[index].attributes['data-md-keyboard-key'].value, char) ? agg.concat(arr[index].attributes['data-md-event-key'].value) : agg;
      }, []);
      typeof newIndex[0] === 'string' && _this.setState(function (state) {
        return {
          listContext: _extends({}, state.listContext, {
            focus: newIndex[0]
          })
        };
      });
    };

    _this.state = {
      id: props.id || uniqueId('cui-list-'),
      last: 0,
      listContext: {
        active: props.active,
        focus: null,
        role: props.itemRole,
        type: props.type
      },
      selectContext: {
        parentKeyDown: _this.handleKeyDown,
        parentOnSelect: _this.handleSelect
      }
    };
    return _this;
  }

  var _proto = List.prototype;

  _proto.componentDidMount = function componentDidMount() {
    var focusFirst = this.props.focusFirst;
    focusFirst && this.listNode && this.determineInitialFocus();
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    var listContext = this.state.listContext;
    if (!this._needsRefocus || !this.listNode) return;

    if (listContext.focus && prevState.listContext.focus !== listContext.focus) {
      this.listNode.querySelector("[data-md-event-key=\"" + listContext.focus + "\"]").focus();
    }
  };

  _proto.getNextFocusedChild = function getNextFocusedChild(current, offset) {
    if (!this.listNode) return null;
    var items = qsa(this.listNode, ".cui-list-item:not(.disabled):not(:disabled):not(.cui-list-item--read-only)");
    var possibleIndex = items.indexOf(current) + offset;

    var getIndex = function getIndex() {
      if (possibleIndex < 0) {
        return items.length - 1;
      } else if (possibleIndex > items.length - 1) {
        return 0;
      } else return possibleIndex;
    };

    this.setState({
      listContext: _extends({}, this.state.listContext, {
        focus: items[getIndex()].attributes['data-md-event-key'].value
      })
    });
  };

  _proto.setFocusToLimit = function setFocusToLimit(target, items, length) {
    var focus = this.state.listContext.focus;
    var newFocusKey = items[target === 'start' ? 0 : length].attributes['data-md-event-key'].value;
    newFocusKey !== focus && this.setState({
      listContext: _extends({}, this.state.listContext, {
        focus: newFocusKey
      })
    });
  };

  _proto.render = function render() {
    var _this2 = this;

    var _this$props = this.props,
        active = _this$props.active,
        children = _this$props.children,
        className = _this$props.className,
        role = _this$props.role,
        tabType = _this$props.tabType,
        wrap = _this$props.wrap,
        props = _objectWithoutPropertiesLoose(_this$props, ["active", "children", "className", "role", "tabType", "wrap"]);

    var _this$state = this.state,
        listContext = _this$state.listContext,
        selectContext = _this$state.selectContext;
    var otherProps = omit(_extends({}, props), ['focusFirst', 'itemRole', 'type']);

    var getActiveId = function getActiveId() {
      var activeNode = active && active.length && _this2.listNode && _this2.listNode.querySelector("[data-md-event-key=\"" + active[0] + "\"]");

      return activeNode && activeNode.id;
    };
    /* eslint-disable jsx-a11y/aria-activedescendant-has-tabindex */


    return React.createElement(SelectableContext.Provider, {
      value: selectContext
    }, React.createElement(ListContext.Provider, {
      value: listContext
    }, React.createElement("div", _extends({
      className: 'cui-list' + ("" + (tabType && " cui-list--" + tabType || '')) + ("" + (wrap && " cui-list--wrap" || '')) + ("" + (className && " " + className || '')),
      role: role,
      "aria-activedescendant": getActiveId(),
      ref: function ref(_ref2) {
        return _this2.listNode = _ref2;
      }
    }, otherProps), React.createElement(UIDReset, null, children))));
    /* eslint-enable*/
  };

  return List;
}(React.Component);

List.propTypes = {
  /** @prop Optional active prop to pass active prop to children | null */
  active: PropTypes.oneOfType([PropTypes.string, PropTypes.array, PropTypes.number]),

  /** @prop Children nodes to render inside List | null */
  children: PropTypes.node,

  /** @prop Optional css class string | '' */
  className: PropTypes.string,

  /** @prop Sets first List item to have focus | true */
  focusFirst: PropTypes.bool,

  /** @prop Optional ID value of List | null */
  id: PropTypes.string,

  /** @prop Optional tabType prop type to manually set child role | 'listitem' */
  itemRole: PropTypes.string,

  /** @prop Callback function invoked by user selecting an interactive item within List | null */
  onSelect: PropTypes.func,

  /** @prop Sets the ARIA role for the Nav, in the context of a TabContainer | 'list' */
  role: PropTypes.string,

  /** @prop Sets the orientation of the List | 'vertical' */
  tabType: PropTypes.oneOf(['vertical', 'horizontal']),

  /** @prop Sets List size | null */
  type: PropTypes.oneOf(['small', 'large', 'space', 'xlarge']),

  /** @prop Optional wrap prop type to wrap items to next row */
  wrap: PropTypes.bool
};
List.defaultProps = {
  active: null,
  children: null,
  className: '',
  id: null,
  itemRole: 'listitem',
  focusFirst: true,
  onSelect: null,
  role: 'list',
  tabType: 'vertical',
  type: null,
  wrap: false
};
List.displayName = 'List';
export default List;

==> react-13.0.0.tgz: package/es/List/index.js <==
function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

/** @component list-item */
import React from 'react';
import PropTypes from 'prop-types';
import omit from 'lodash/omit';
import uniqueId from 'lodash/uniqueId';
import qsa from 'dom-helpers/query/querySelectorAll';
import { UIDReset } from 'react-uid';
import SelectableContext from "../SelectableContext";
import ListContext from "../ListContext";

var List =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(List, _React$Component);

  List.getDerivedStateFromProps = function getDerivedStateFromProps(_ref, state) {
    var active = _ref.active;
    return active ? _extends({}, state, {
      listContext: _extends({}, state.listContext, {
        active: active
      })
    }) : state;
  };

  function List(props) {
    var _this;

    _this = _React$Component.call(this, props) || this;

    _this.determineInitialFocus = function () {
      var items = qsa(_this.listNode, ".cui-list-item:not(.disabled):not(:disabled):not(.cui-list-item--read-only)");
      _this._needsRefocus = true;
      items.length && _this.getNextFocusedChild(items[0], 0);
    };

    _this.getIncludesFirstCharacter = function (str, char) {
      return str.charAt(0).toLowerCase().includes(char);
    };

    _this.getValue = function (arr, index, attribute) {
      return arr[index].attributes["data-md-" + attribute + "-key"] && arr[index].attributes["data-md-" + attribute + "-key"].value;
    };

    _this.handleKeyDown = function (e) {
      var shouldFocusActive = _this.props.shouldFocusActive;
      var focus = _this.state.listContext.focus;
      var clickEvent;
      var tgt = e.currentTarget;
      var char = e.key;
      var items = qsa(_this.listNode, ".cui-list-item:not(.disabled):not(:disabled):not(.cui-list-item--read-only)");
      var length = items.length && items.length - 1 || 0;
      var focusIdx = focus && items.indexOf(_this.listNode.querySelector("[data-md-event-key=\"" + focus + "\"]")) || 0;
      var flag = false;

      var isPrintableCharacter = function isPrintableCharacter(str) {
        return str.length === 1 && str.match(/\S/);
      };

      switch (e.which) {
        case 9:
          if (shouldFocusActive) {
            _this._needsRefocus = false;

            _this.setFocusToActive();
          }

          break;

        case 32:
        case 13:
          try {
            clickEvent = new MouseEvent('click', {
              view: window,
              bubbles: true,
              cancelable: true
            });
          } catch (err) {
            if (document.createEvent) {
              // DOM Level 3 for IE 9+
              clickEvent = document.createEvent('MouseEvents');
              clickEvent.initEvent('click', true, true);
            }
          }

          tgt.dispatchEvent(clickEvent);
          flag = true;
          break;

        case 38:
        case 37:
          _this.getNextFocusedChild(tgt, -1);

          _this._needsRefocus = true;
          flag = true;
          break;

        case 39:
        case 40:
          _this.getNextFocusedChild(tgt, 1);

          _this._needsRefocus = true;
          flag = true;
          break;

        case 33:
        case 36:
          _this.setFocusToLimit('start', items, length);

          _this._needsRefocus = true;
          flag = true;
          break;

        case 34:
        case 35:
          _this.setFocusToLimit('end', items, length);

          _this._needsRefocus = true;
          flag = true;
          break;

        default:
          if (isPrintableCharacter(char)) {
            _this.setFocusByFirstCharacter(char, focusIdx, items, length);

            _this._needsRefocus = true;
            flag = true;
          }

          break;
      }

      if (flag) {
        e.stopPropagation();
        e.preventDefault();
      }
    };

    _this.handleSelect = function (e, opts) {
      var _this$props = _this.props,
          onSelect = _this$props.onSelect,
          trackActive = _this$props.trackActive;
      var active = _this.state.listContext.active;
      var eventKey = opts.eventKey,
          label = opts.label,
          value = opts.value;
      var items = qsa(_this.listNode, '.cui-list-item');
      var index = items.indexOf(_this.listNode.querySelector("[data-md-event-key=\"" + eventKey + "\"]"));

      _this.setFocus(index); // Don't do anything if onSelect Event Handler is present


      if (onSelect) {
        return onSelect(e, {
          eventKey: _this.getValue(items, index, 'event'),
          label: label,
          value: value
        });
      } // Don't do anything if index is the same or outside of the bounds


      if (eventKey === active || index < 0 || index > items.length - 1) return; // Keep reference to last index for event handler

      var last = active; // Call change event handler

      trackActive && _this.setState(function (state) {
        return {
          last: last,
          listContext: _extends({}, state.listContext, {
            active: _this.getValue(items, index, 'event')
          })
        };
      });
    };

    _this.setFocus = function (index) {
      var items = qsa(_this.listNode, '.cui-list-item');

      _this.setState(function (state) {
        return {
          listContext: _extends({}, state.listContext, {
            focus: _this.getValue(items, index, 'event')
          })
        };
      });
    };

    _this.setFocusByFirstCharacter = function (char, focusIdx, items, length) {
      var listContext = _this.state.listContext;
      var newIndex = items.reduce(function (agg, item, idx, arr) {
        var index = focusIdx + idx + 1 > length ? Math.abs(focusIdx + idx - length) : focusIdx + idx + 1;
        return !agg.length && _this.getValue(arr, index, 'keyboard') && _this.getIncludesFirstCharacter(_this.getValue(arr, index, 'keyboard'), char) ? agg.concat(_this.getValue(arr, index, 'event')) : agg;
      }, []);
      typeof newIndex[0] === 'string' && listContext.focus !== newIndex[0] && _this.setState(function (state) {
        return {
          listContext: _extends({}, state.listContext, {
            focus: newIndex[0]
          })
        };
      });
    };

    _this.state = {
      id: props.id || uniqueId('cui-list-'),
      last: 0,
      listContext: {
        active: props.active,
        focus: null,
        role: props.itemRole,
        type: props.type
      },
      selectContext: {
        parentKeyDown: _this.handleKeyDown,
        parentOnSelect: _this.handleSelect
      }
    };
    return _this;
  }

  var _proto = List.prototype;

  _proto.componentDidMount = function componentDidMount() {
    var focusFirst = this.props.focusFirst;
    focusFirst && this.listNode && this.determineInitialFocus();
  };

  _proto.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    var listContext = this.state.listContext;
    if (!this._needsRefocus || !this.listNode) return;

    if (listContext.focus && prevState.listContext.focus !== listContext.focus) {
      this.listNode.querySelector("[data-md-event-key=\"" + listContext.focus + "\"]").focus();
    }
  };

  _proto.getNextFocusedChild = function getNextFocusedChild(current, offset) {
    if (!this.listNode) return null;
    var shouldLoop = this.props.shouldLoop;
    var listContext = this.state.listContext;
    var items = qsa(this.listNode, ".cui-list-item:not(.disabled):not(:disabled):not(.cui-list-item--read-only)");
    var possibleIndex = items.indexOf(current) + offset;

    var getIndex = function getIndex() {
      if (possibleIndex < 0) {
        return shouldLoop ? items.length - 1 : 0;
      } else if (possibleIndex > items.length - 1) {
        return shouldLoop ? 0 : items.length - 1;
      } else return possibleIndex;
    };

    return listContext.focus !== this.getValue(items, getIndex(), 'event') && this.setState({
      listContext: _extends({}, listContext, {
        focus: this.getValue(items, getIndex(), 'event')
      })
    });
  };

  _proto.setFocusToActive = function setFocusToActive() {
    this.setState({
      listContext: _extends({}, this.state.listContext, {
        focus: this.state.listContext.active
      })
    });
  };

  _proto.setFocusToLimit = function setFocusToLimit(target, items, length) {
    var focus = this.state.listContext.focus;
    var index = target === 'start' ? 0 : length;
    var newFocusKey = this.getValue(items, index, 'event');
    newFocusKey !== focus && this.setState({
      listContext: _extends({}, this.state.listContext, {
        focus: newFocusKey
      })
    });
  };

  _proto.render = function render() {
    var _this2 = this;

    var _this$props2 = this.props,
        active = _this$props2.active,
        children = _this$props2.children,
        className = _this$props2.className,
        role = _this$props2.role,
        tabType = _this$props2.tabType,
        wrap = _this$props2.wrap,
        props = _objectWithoutPropertiesLoose(_this$props2, ["active", "children", "className", "role", "tabType", "wrap"]);

    var _this$state = this.state,
        listContext = _this$state.listContext,
        selectContext = _this$state.selectContext;
    var otherProps = omit(_extends({}, props), ['focusFirst', 'itemRole', 'shouldFocusActive', 'shouldLoop', 'trackActive', 'type']);

    var getActiveId = function getActiveId() {
      var activeNode = active && active.length && _this2.listNode && _this2.listNode.querySelector("[data-md-event-key=\"" + active[0] + "\"]");

      return activeNode && activeNode.id;
    };
    /* eslint-disable jsx-a11y/aria-activedescendant-has-tabindex */


    return React.createElement(SelectableContext.Provider, {
      value: selectContext
    }, React.createElement(ListContext.Provider, {
      value: listContext
    }, React.createElement("div", _extends({
      className: 'cui-list' + ("" + (tabType && " cui-list--" + tabType || '')) + ("" + (wrap && " cui-list--wrap" || '')) + ("" + (className && " " + className || '')),
      role: role,
      "aria-activedescendant": getActiveId(),
      ref: function ref(_ref2) {
        return _this2.listNode = _ref2;
      }
    }, otherProps), React.createElement(UIDReset, null, children))));
    /* eslint-enable*/
  };

  return List;
}(React.Component);

List.propTypes = {
  /** @prop Optional active prop to pass active prop to children | null */
  active: PropTypes.oneOfType([PropTypes.string, PropTypes.array, PropTypes.number]),

  /** @prop Children nodes to render inside List | null */
  children: PropTypes.node,

  /** @prop Optional css class string | '' */
  className: PropTypes.string,

  /** @prop Sets first List item to have focus | true */
  focusFirst: PropTypes.bool,

  /** @prop Optional ID value of List | null */
  id: PropTypes.string,

  /** @prop Optional tabType prop type to manually set child role | 'listitem' */
  itemRole: PropTypes.string,

  /** @prop Callback function invoked by user selecting an interactive item within List | null */
  onSelect: PropTypes.func,

  /** @prop Sets the ARIA role for the Nav, in the context of a TabContainer | 'list' */
  role: PropTypes.string,

  /** @prop Determines if focus should revert to active on list exit | false */
  shouldFocusActive: PropTypes.bool,

  /** @prop Determines if keyboard navigation should loop through list | true */
  shouldLoop: PropTypes.bool,

  /** @prop Sets the orientation of the List | 'vertical' */
  tabType: PropTypes.oneOf(['vertical', 'horizontal']),

  /** @prop Determines if List wrapper should track active | true */
  trackActive: PropTypes.bool,

  /** @prop Sets List size | null */
  type: PropTypes.oneOf(['small', 'large', 'space', 'xlarge']),

  /** @prop Optional wrap prop type to wrap items to next row */
  wrap: PropTypes.bool
};
List.defaultProps = {
  active: null,
  children: null,
  className: '',
  id: null,
  itemRole: 'listitem',
  focusFirst: true,
  onSelect: null,
  role: 'list',
  shouldFocusActive: false,
  shouldLoop: true,
  tabType: 'vertical',
  trackActive: true,
  type: null,
  wrap: false
};
List.displayName = 'List';
export default List;
